/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import static org.eclipse.emf.ecore.ETypedElement.UNBOUNDED_MULTIPLICITY;
import static org.eclipse.emf.ecore.ETypedElement.UNSPECIFIED_MULTIPLICITY;

import java.util.Collection;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

/**
 * Base class for dialogs to select/edit values of multi-valued annotations.
 * 
 * @author barner
 */
public abstract class MultiValueAnnotationDialogBase<T> extends Dialog
		implements IMultiValueAnnotationDialog<T> {

	/** Lower multiplicity bound. */
	private int lowerMultiplicityBound;

	/** Upper multiplicity bound. */
	private int upperMultiplicityBound;

	/** Name of action performed in this dialog. */
	private String actionName;

	/** The collection of elements selected/edited using this dialog. */
	private Collection<T> elements;

	/** Constructs a new {@link MultiValueAnnotationDialogBase}. */
	public MultiValueAnnotationDialogBase(String actionName, int lowerMultiplicityBound,
			int upperMultiplicityBound) {
		super((Shell)null);
		this.actionName = actionName;
		this.lowerMultiplicityBound = lowerMultiplicityBound;
		this.upperMultiplicityBound = upperMultiplicityBound;
	}

	/** {@inheritDoc} */
	@Override
	protected final Control createButtonBar(Composite parent) {
		Control ctrl = super.createButtonBar(parent);
		updateOkButton();

		return ctrl;
	}

	/** Helper to select the right number for "element". */
	private String titleElementStringHelper(int n) {
		return n != 1 ? "elements" : "element";
	}

	/** {@inheritDoc} */
	@Override
	protected final void configureShell(Shell newShell) {
		super.configureShell(newShell);
		String title;
		if(lowerMultiplicityBound == UNSPECIFIED_MULTIPLICITY || lowerMultiplicityBound == 0) {
			if(upperMultiplicityBound == UNSPECIFIED_MULTIPLICITY ||
					upperMultiplicityBound == UNBOUNDED_MULTIPLICITY) {
				title = actionName + " multiple elements";
			} else {
				title = actionName + " at most " + upperMultiplicityBound + " " +
						titleElementStringHelper(upperMultiplicityBound);
			}
		} else {
			if(upperMultiplicityBound == UNSPECIFIED_MULTIPLICITY ||
					upperMultiplicityBound == UNBOUNDED_MULTIPLICITY) {
				title = actionName + " at least " + lowerMultiplicityBound + " " +
						titleElementStringHelper(lowerMultiplicityBound);
			} else {
				title = actionName + " " + lowerMultiplicityBound + " to " +
						upperMultiplicityBound + " " +
						titleElementStringHelper(upperMultiplicityBound);
			}
		}
		newShell.setText(title);
	}

	/** Enables the Ok button iff the multiplicity constraints are met. */
	protected final void updateOkButton() {
		Button okButton = getButton(IDialogConstants.OK_ID);

		int numElements = getElementsFromDialog().size();
		boolean lowerBoundOk = lowerMultiplicityBound == UNSPECIFIED_MULTIPLICITY ||
				numElements >= lowerMultiplicityBound;
		boolean upperBoundOk = upperMultiplicityBound == UNSPECIFIED_MULTIPLICITY ||
				upperMultiplicityBound == UNBOUNDED_MULTIPLICITY ||
				numElements <= upperMultiplicityBound;
		okButton.setEnabled(lowerBoundOk && upperBoundOk);
	}

	/** {@inheritDoc} */
	@Override
	protected final void okPressed() {
		elements = getElementsFromDialog();

		super.okPressed();
	}

	/** {@inheritDoc} */
	@Override
	public final Collection<T> getElements() {
		return elements;
	}

	/** Returns the current {@link Collection} of elements from the corresponding widget of the editor. */
	protected abstract Collection<T> getElementsFromDialog();
}
