/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.annotation.valueprovider.EStructuralFeatureDescriptor;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.INamedElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Base class for {@link EditingSupport}s for annotations that can hold multiple values.
 * 
 * @author barner
 */
public abstract class MultiValueAnnotationEditingSupportBase<T> extends AnnotationEditingSupportBase
		implements IMultiValueAnnotationEditingSupport<T> {

	/** The underlying annotation */
	protected IAnnotatedSpecification specification;

	/** Annotation's {@link EStructuralFeatureDescriptor}. */
	protected EStructuralFeatureDescriptor eStructuralFeatureDescriptor;

	/** Constructs a new {@link MultiValueAnnotationEditingSupportBase}. */
	public MultiValueAnnotationEditingSupportBase(ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> clazz, IAnnotatedSpecification specification,
			EStructuralFeatureDescriptor eStructuralFeatureDescriptor) {
		super(viewer, clazz);

		this.specification = specification;
		this.eStructuralFeatureDescriptor = eStructuralFeatureDescriptor;
	}

	/** {@inheritDoc} */
	@Override
	protected CellEditor getCellEditor(Object element) {

		if(!(element instanceof AnnotationEntry)) {
			return null;
		}
		AnnotationEntry entry = (AnnotationEntry)element;

		Object value = entry.getSpecificationValue(specClass);
		if(!(value instanceof Collection<?>)) {
			return null;
		}

		Composite parent = (Composite)getViewer().getControl();
		@SuppressWarnings("unchecked") IMultiValueAnnotationDialog<T> dialog =
				createMultiEditingDialog(parent, entry.getModelElement(), (Collection<T>)value);

		return new MultiValueAnnotationCellEditor(parent, dialog);
	}

	/** {@inheritDoc} */
	@Override
	protected boolean canEdit(Object element) {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	protected Object getValue(Object element) {
		return getLabel(element);
	}

	/** {@inheritDoc} */
	@Override
	public String getLabel(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry entry = (AnnotationEntry)element;

			Object value = entry.getSpecificationValue(specClass);

			if(value instanceof Collection<?>) {
				StringBuffer valueCollectionString = new StringBuffer();
				for(Iterator<?> iter = ((Collection<?>)value).iterator(); iter.hasNext();) {
					Object v = iter.next();
					if(v instanceof INamedElement) {
						valueCollectionString.append(((INamedElement)v).getName());
					} else {
						valueCollectionString.append(v != null ? v.toString() : "<null>");
					}
					if(iter.hasNext()) {
						valueCollectionString.append(", ");
					}
				}
				return "[" + valueCollectionString + "]";
			}
		}
		return "[]";
	}

	/** {@inheritDoc} */
	@Override
	protected void setValue(Object element, final Object value) {
		// Writes back into the model the collection of selected elements
		if(element instanceof AnnotationEntry && value instanceof Collection<?>) {
			final AnnotationEntry entry = (AnnotationEntry)element;

			ITopLevelElement modelContext = IPersistencyService.getInstance()
					.getTopLevelElementFor(entry.getModelElement());
			modelContext.runAsCommand(new Runnable() {

				@Override
				public void run() {
					try {
						entry.setSpecificationValue(value, specClass);
					} catch(Exception e) {
						return;
					}
				}
			});
		}
	}

	/** {@link CellEditor} that opens a dialog to edit a multi-valued annotation. */
	private class MultiValueAnnotationCellEditor extends DialogCellEditor {

		/** The dialog opened by this cell editor. */
		private IMultiValueAnnotationDialog<T> dialog;

		/** Constructor. */
		public MultiValueAnnotationCellEditor(Composite parent,
				IMultiValueAnnotationDialog<T> dialog) {
			super(parent);

			this.dialog = dialog;
		}

		/** {@inheritDoc} */
		@Override
		protected Object openDialogBox(Control cellEditorWindow) {

			dialog.open();
			return dialog.getElements();
		}
	}
}
