/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor;

import static org.eclipse.jface.resource.FontDescriptor.createFrom;

import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.base.model.element.ConstraintConfiguration;
import org.fortiss.tooling.kernel.extension.IConstraint;

// TODO Refactor together with ConstraintBasedProcessEditor and provide a dedicated perspective (see #3189)
/**
 * This class is a helper class that provides, 1) the colors for the constraint groups,
 * 2) manage the references between the constraint configurations and
 * 3) manages the tree manipulation that is used to visualize the constraintsGroups,
 * constraintConfigurations and constraints.
 * 
 * @author abid
 * 
 */
public class ConstraintBasedProcessEditorHelper {

	/** Color Array for at least seven groups. */
	private static int[] colorArray =
			{SWT.COLOR_DARK_GREEN, SWT.COLOR_DARK_BLUE, SWT.COLOR_DARK_YELLOW, SWT.COLOR_BLACK,
					SWT.COLOR_DARK_RED, SWT.COLOR_GREEN, SWT.COLOR_CYAN};

	/** Counter for the colorArray */
	private static int colorCtr = 0;

	/** Reference to a constraint configuration. */
	private static class WithContextualConfiguration {

		/** Contextual constraint configuration. */
		ConstraintConfiguration config;

		/** Constructor. */
		public WithContextualConfiguration(ConstraintConfiguration config) {
			this.config = config;
		}
	}

	/** Constraint accompanied with a configuration providing the context. */
	public static class ConstraintWithContextualConfiguration extends WithContextualConfiguration {

		/** Constraint. */
		/* package */Class<? extends IConstraint> cstr;

		/** Constructor. */
		public ConstraintWithContextualConfiguration(Class<? extends IConstraint> cstr,
				ConstraintConfiguration config) {
			super(config);
			this.cstr = cstr;
		}
	}

	/**
	 * Configuration accompanied with another configuration providing the context.
	 * Intent is that the contained configuration is a dependency of the context one.
	 */
	public static class ConfigurationRefWithContextualConfiguration
			extends WithContextualConfiguration {

		/** Constraint. */
		/* package */ConstraintConfiguration target;

		/** Constructor. */
		public ConfigurationRefWithContextualConfiguration(ConstraintConfiguration configRef,
				ConstraintConfiguration config) {
			super(config);
			this.target = configRef;
		}

		/** {@inheritDoc} */
		@Override
		public boolean equals(Object obj) {
			if(obj instanceof ConfigurationRefWithContextualConfiguration) {
				// current target name
				String currentTargetName = this.target.getName();
				// contextualRef Target of the configuration
				String configRefTrgtObjName =
						((ConfigurationRefWithContextualConfiguration)obj).target.getName();
				// current configuration
				String currentConfigName = this.config.getName();
				// contextualRef Object of the configuration
				String configRefObjName =
						((ConfigurationRefWithContextualConfiguration)obj).config.getName();
				return currentTargetName.equalsIgnoreCase(configRefTrgtObjName) &&
						currentConfigName.equalsIgnoreCase(configRefObjName);
			}
			return false;
		}

		/** {@inheritDoc} */
		@Override
		public int hashCode() {
			return target.hashCode();
		}
	}

	/** Class to manipulate and store parent of elements. */
	public static class TreeStructureNode {

		/** Parent object variable. */
		/* package */Object parent;

		/** Data object variable. */
		/* package */Object data;

		/** Constructor. */
		public TreeStructureNode(Object parent, Object children) {
			this.parent = parent;
			this.data = children;
		}

		/** {@inheritDoc} */
		@Override
		public boolean equals(Object obj) {
			if(obj instanceof TreeStructureNode) {
				// the current tree node to be compared with
				String currentStructureNode = this.data.toString();
				// the tree node to be checked
				String treeStructureNode = ((TreeStructureNode)obj).data.toString();
				// parent of the current tree node
				String currentParentNode = this.parent.toString();
				// parent of the object that needs to be checked
				String objectParentNode = ((TreeStructureNode)obj).parent.toString();
				return currentStructureNode.equalsIgnoreCase(treeStructureNode) &&
						currentParentNode.equalsIgnoreCase(objectParentNode);
			}
			return false;
		}

		/** {@inheritDoc} */
		@Override
		public int hashCode() {
			return parent.hashCode();
		}
	}

	/** Styler class to create colored bold labels for constraint group name. */
	public static class NormalFontStyler extends Styler {

		/** Counter to keep track of currently used color in color array. */
		private int colorIndex = 0;

		/** Constructor. */
		public NormalFontStyler() {
			this.colorIndex = colorCtr;
		}

		/** {@inheritDoc} */
		@Override
		public void applyStyles(final TextStyle textStyle) {

			FontDescriptor boldDescriptor = createFrom(new FontData()).setStyle(SWT.BOLD);
			Font boldFont = boldDescriptor.createFont(Display.getCurrent());
			textStyle.font = boldFont;
			textStyle.foreground = Display.getCurrent().getSystemColor(colorArray[colorIndex]);
		}
	}

	/** Styler class to create red color labels. */
	public static class RedFontStyler extends Styler {
		/** {@inheritDoc} */
		@Override
		public void applyStyles(final TextStyle textStyle) {
			FontDescriptor boldDescriptor = createFrom(new FontData()).setStyle(SWT.BOLD);
			Font boldFont = boldDescriptor.createFont(Display.getCurrent());
			textStyle.font = boldFont;
			textStyle.foreground = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_RED);
		}
	}

	/** Styler class to create green color labels. */
	public static class GreenFontStyler extends Styler {
		/** {@inheritDoc} */
		@Override
		public void applyStyles(final TextStyle textStyle) {
			FontDescriptor boldDescriptor = createFrom(new FontData()).setStyle(SWT.BOLD);
			Font boldFont = boldDescriptor.createFont(Display.getCurrent());
			textStyle.font = boldFont;
			textStyle.foreground = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GREEN);
		}
	}

	/**
	 * This function is responsible for updating the color of the constraints groups as they are
	 * added by updating the color counter (i.e., ColorCtr)
	 * 
	 * @param groupSize
	 *            is the size of the constraints groups
	 */
	public static void adjustColorIndex(int groupSize) {
		colorCtr++;
		colorCtr %= groupSize;
	}
}
