/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.annotation.valueprovider;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
import org.fortiss.tooling.base.annotation.valueprovider.EStructuralFeatureDescriptor.EReferenceScope;
import org.fortiss.tooling.base.model.element.IDerivedAnnotation;

/**
 * Base class for {@link IAnnotationValueProvider} for derived annotations (i.e., based on
 * {@link IDerivedAnnotation}).
 * 
 * @author barner
 */
public abstract class DerivedAnnotationValueProviderBase<T extends IDerivedAnnotation<?>>
		extends EStructuralFeatureValueProviderBase<T> {

	/** Flag indicating whether the derived value shall be editable by the user. */
	private boolean isEditableByUser;

	/** Constructor. */
	public DerivedAnnotationValueProviderBase(EClass annotatedSpecificationEClass,
			EAttribute eAttribute, boolean isEditableByUser) {

		super(annotatedSpecificationEClass, new EStructuralFeatureDescriptor(eAttribute));
		this.isEditableByUser = isEditableByUser;
	}

	/** Constructor. */
	public DerivedAnnotationValueProviderBase(EClass annotatedSpecificationEClass,
			EReference eReference, EReferenceScope eReferenceScope, boolean isEditableByUser) {

		super(annotatedSpecificationEClass,
				new EStructuralFeatureDescriptor(eReference, eReferenceScope));
		this.isEditableByUser = isEditableByUser;
	}

	/** Constructor. */
	public DerivedAnnotationValueProviderBase(EClass annotatedSpecificationEClass,
			EAttribute eAttribute) {

		this(annotatedSpecificationEClass, eAttribute, false);
	}

	/** Constructor. */
	public DerivedAnnotationValueProviderBase(EClass annotatedSpecificationEClass,
			EReference eReference, EReferenceScope eReferenceScope) {

		this(annotatedSpecificationEClass, eReference, eReferenceScope, false);
	}

	/** Constructor. */
	public DerivedAnnotationValueProviderBase(EClass annotatedSpecificationEClass) {
		super(annotatedSpecificationEClass, new EStructuralFeatureDescriptor(null));
		this.isEditableByUser = false;
	}

	/** Predicate whether the derived value shall be editable by the user. */
	public boolean isEditableByUser() {
		return isEditableByUser;
	}

	/** {@inheritDoc} */
	@Override
	public boolean canEdit(T specification) {
		// Deactivate editing if...
		if(!isEditableByUser) {
			// ...editing is principally disabled or...
			return false;
		}

		boolean preferDerivedValue =
				isEditableByUser && !specification.isUserAnnotatedValuePreferred();
		boolean derivedValueAvailable = specification.getDerivedValue() != null;

		if(preferDerivedValue && derivedValueAvailable) {
			// ...the derived value is available and it should override the user annotated one.
			return false;
		}
		return super.canEdit(specification);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public <U> U getAnnotationValue(T specification) {
		U calculatedValue = (U)specification.getValue();
		if(calculatedValue != null) {
			return calculatedValue;
		}

		return super.getAnnotationValue(specification);
	}

	/** {@inheritDoc} */
	@Override
	public <U> void setAnnotationValue(U value, T specification) throws Exception {
		if(specification.getDerivedFeature() != null) {
			if(value instanceof String) {
				setAnnotationValueFromString((String)value, specification);
			} else {
				specification.eSet(specification.getDerivedFeature(), value);
			}
			return;
		}

		super.setAnnotationValue(value, specification);
	}
}
